------------------------------ MODULE abpMain ------------------------------
(* This is an attempt to modularise the abpPar Module.

The two process used to specify a Channel that can non deterministicly but fairly send or drop a packet 
are in the dataWire Module. This has been instantiated twice, first for sending data and second for 
sending the acknowlaements.  The ability to drop acknowlagments is a slight extension on the abpPar spec.
The two processes that make up the sender are in the module abpSender and the two process for the 
receiver are in abpReceiver.

A commom Process Algebra, PA, approach is to implement a system by building several component process 
that comunicate both with each other and with the outside world. 
Then by hiding all the internal communication we have a system that only describes the observable behavior
of the system. 
   This example is: given two faulty communication Channels we define two processes, the Sender 
and the Receive that use these Channels in such a way that to the out side worls it looks like they are 
connected by a fault free one place buffer.

Here we give abpMain our modularised implemetation of the Alternating Bit Protocol its main weakness is that we 
cannot find how to define the fairness properties! 

We define a one place buffer in the module OnePlaceBufferSpec and show that abpMain is a refinement of 
 OnePlaceBufferSpec. In addition we can add the fairness condition to OnePlaceBufferSpec and show that it 
 satisfies the liveness property
                  Fin == <>[](messOut = Message)
It would be good to be able to specify fairness fo abpMain so that we could check that abpMain+fair is a
 refinement of OnePlaceBufferSpec+fair                
                  
*)
 
EXTENDS Naturals, Integers, TLC, Sequences, Bags, FiniteSets
CONSTANT   NULL, NL, Message  \* to be set in model  to a sequence
VARIABLES sendDataQueue, 
          receiveDataQueue,
          sendAckQueue,   \* acknowlage channel <<bit>>
          receiveAckQueue,    \* acknowlage channel <<bit>>
          mess,           \* message input to  sender process
          messOut,        \* message output from receiver process
          \* idealy the following data would be private to sub-modules
          sender_bit,
          rec_bit,
          pc1,pc2,pc3,pc4  
VARIABLES     read_state     \* ghost variable
          
 vars == <<sendDataQueue,  receiveDataQueue, sendAckQueue, receiveAckQueue, 
           mess,messOut, sender_bit, rec_bit, pc1,pc2,pc3,pc4, read_state >>     
----------------------------------
Sender   == INSTANCE abpSender WITH  pc <-pc1
Receiver == INSTANCE abpReceiver WITH pc <-pc2 
dataWir  == INSTANCE dataWire WITH pc <- pc3
ackWir   == INSTANCE dataWire WITH sendDataQueue <- receiveAckQueue, receiveDataQueue <- sendAckQueue, pc <- pc4
----------------------------------
(* Used for refinement check *)
OPB == INSTANCE OnePlaceBufferSpec  
----------------------------------
  
Init ==  /\ Receiver!Init 
         /\ Sender!Init 
         /\ dataWir!Init 
         /\ ackWir!Init
         /\ mess = Message
         /\ messOut = <<>>
         /\ sender_bit =  0
         /\ rec_bit = 1 
         /\ receiveDataQueue = <<>>
         /\ sendDataQueue = <<>>
         /\ receiveAckQueue = <<>>
         /\ sendAckQueue = <<>>
         /\ read_state = TRUE    \* ghost variable
                
         
sendData == /\ Sender!sendData
            /\ UNCHANGED <<pc2,pc3,pc4, rec_bit,messOut, receiveAckQueue, receiveDataQueue >> 

receiveData == /\ Receiver!receiveData
               /\ UNCHANGED <<pc1,pc3,pc4, sender_bit,mess, sendAckQueue, sendDataQueue>> 

dataChannel ==  /\  dataWir!Next
                /\  UNCHANGED <<read_state,mess,messOut, pc1, pc2, pc4, receiveAckQueue, sendAckQueue, rec_bit, sender_bit>>         

ackChannel ==  /\  ackWir!Next
               /\  UNCHANGED <<read_state,mess,messOut, pc1, pc2, pc3, receiveDataQueue, sendDataQueue, rec_bit,sender_bit>>         
                
------------------------------------   
      
Next == receiveData \/ sendData \/ dataChannel \/ ackChannel

Spec == Init /\ [][Next]_vars
        (*      /\ SF_vars(Sender!sendData)
             /\ SF_vars(Receiver!receiveData)
             /\ SF_vars(Sender!sendAck)
             /\ SF_vars(Receiver!receiveAck)
             /\ SF_vars(dataWir!dataChannel)
             /\ SF_vars(dataWir!dropChannel)
             /\ SF_vars(ackWir!dataChannel)
             /\ SF_vars(ackWir!dropChannel)   *)
             
        (*     /\ SF_vars(Sender!sSdata)
             /\ SF_vars(Sender!sSack)
             /\ SF_vars(Receiver!rRdata)
             /\ SF_vars(Receiver!rRdata)
             /\ SF_vars(dataWir!data)
             /\ SF_vars(dataWir!drop)
             /\ SF_vars(ackWir!data)
             /\ SF_vars(ackWir!drop) *)
        
Fin == <>[](messOut = Message)
\* model checked at every state

                

=============================================================================
\* Modification History
\* Last modified Sun Feb 17 13:17:23 NZDT 2019 by dstr
\* Created Thu Feb 14 15:38:49 NZDT 2019 by dstr
