------------------------------ MODULE abpReceiver ------------------------------
(* This code implements the ABP as
 two processes a sending process and a receiving process that comubicate by 
 sharing a dataQueue and an, acknowlage,  ackQueue
*)

EXTENDS Naturals, Integers, TLC, Sequences, Bags, FiniteSets
CONSTANTS   NULL, NL, Message  \* to be set in model  to a sequence

(* --fair algorithm trade
variables  receiveDataQueue = <<>>,
           receiveAckQueue = <<>>,    \* acknowlage channel <<bit>>
           messOut  = <<>>,    \* message output from receiver process
           rec_bit = 1,
           read_state = TRUE;  \* ghost variable


fair+ process receiveData = 21 \* first process to shut down
 begin 
rD:
  while Len(messOut) < Len(Message) do  \* could use TRUE  
    rRdata:
    await (Len(receiveDataQueue) > 0);
       if rec_bit /= Head(receiveDataQueue).bit then        
         messOut := Append(messOut, Head(receiveDataQueue).data);        
         rec_bit := Head(receiveDataQueue).bit;
         receiveDataQueue := Tail(receiveDataQueue); 
         receiveAckQueue := <<rec_bit>>;
         read_state := FALSE;
       else
          receiveDataQueue := Tail(receiveDataQueue); 
       end if;                                 
  end while;     
    print ("RecaData END " \o ToString(pc));
end process;

fair+ process receiveAck = 22 \* first process to shut down
 begin 
rA:+
  while Len(messOut) < Len(Message) do  \* could use TRUE  
    rRack:
    await (Len(receiveAckQueue) = 0);    
    receiveAckQueue := <<rec_bit>>;
  end while;   
end process;


end algorithm; *)


\* BEGIN TRANSLATION
VARIABLES receiveDataQueue, receiveAckQueue, messOut, rec_bit, read_state, pc

vars == << receiveDataQueue, receiveAckQueue, messOut, rec_bit, read_state, 
           pc >>

ProcSet == {21} \cup {22}

Init == (* Global variables *)
        /\ receiveDataQueue = <<>>
        /\ receiveAckQueue = <<>>
        /\ messOut = <<>>
        /\ rec_bit = 1
        /\ read_state = TRUE
        /\ pc = [self \in ProcSet |-> CASE self = 21 -> "rD"
                                        [] self = 22 -> "rA"]

rD == /\ pc[21] = "rD"
      /\ IF Len(messOut) < Len(Message)
            THEN /\ pc' = [pc EXCEPT ![21] = "rRdata"]
            ELSE /\ PrintT(("RecaData END " \o ToString(pc)))
                 /\ pc' = [pc EXCEPT ![21] = "Done"]
      /\ UNCHANGED << receiveDataQueue, receiveAckQueue, messOut, rec_bit, 
                      read_state >>

rRdata == /\ pc[21] = "rRdata"
          /\ (Len(receiveDataQueue) > 0)
          /\ IF rec_bit /= Head(receiveDataQueue).bit
                THEN /\ messOut' = Append(messOut, Head(receiveDataQueue).data)
                     /\ rec_bit' = Head(receiveDataQueue).bit
                     /\ receiveDataQueue' = Tail(receiveDataQueue)
                     /\ receiveAckQueue' = <<rec_bit'>>
                     /\ read_state' = FALSE
                ELSE /\ receiveDataQueue' = Tail(receiveDataQueue)
                     /\ UNCHANGED << receiveAckQueue, messOut, rec_bit, 
                                     read_state >>
          /\ pc' = [pc EXCEPT ![21] = "rD"]

receiveData == rD \/ rRdata

rA == /\ pc[22] = "rA"
      /\ IF Len(messOut) < Len(Message)
            THEN /\ pc' = [pc EXCEPT ![22] = "rRack"]
            ELSE /\ pc' = [pc EXCEPT ![22] = "Done"]
      /\ UNCHANGED << receiveDataQueue, receiveAckQueue, messOut, rec_bit, 
                      read_state >>

rRack == /\ pc[22] = "rRack"
         /\ (Len(receiveAckQueue) = 0)
         /\ receiveAckQueue' = <<rec_bit>>
         /\ pc' = [pc EXCEPT ![22] = "rA"]
         /\ UNCHANGED << receiveDataQueue, messOut, rec_bit, read_state >>

receiveAck == rA \/ rRack

Next == receiveData \/ receiveAck
           \/ (* Disjunct to prevent deadlock on termination *)
              ((\A self \in ProcSet: pc[self] = "Done") /\ UNCHANGED vars)

Spec == /\ Init /\ [][Next]_vars
        /\ WF_vars(Next)
        /\ SF_vars(receiveData)
        /\ SF_vars(receiveAck)

Termination == <>(\A self \in ProcSet: pc[self] = "Done")

\* END TRANSLATION

Fin == <>[](messOut = Message)
\* model checked at every state

(*
MyTermination == Termination /\  <>(\A self \in ProcSet: pc[self] = "X" )
MySpec == Spec /\ SF_vars(data)
               /\ SF_vars(drop)
               /\ SF_vars(sSdata)
               /\ SF_vars(sSack)
               /\ SF_vars(rRdata)
               /\ SF_vars(rRack)
*)                







=============================================================================
\* Modification History
\* Last modified Sun Feb 17 10:01:15 NZDT 2019 by dstr
\* Created Thu Feb 14 15:38:49 NZDT 2019 by dstr
