------------------------------ MODULE abpSender ------------------------------
(* This code implements the ABP as
 two processes a sending process and a receiving process that comubicate by 
 sharing a dataQueue and an, acknowlage,  ackQueue


*)

EXTENDS Naturals, Integers, TLC, Sequences, Bags, FiniteSets
CONSTANTS   NULL,NL, Message  \* to be set in model  to a sequence

(* --fair algorithm trade
variables sendDataQueue  = <<>>, \* public <<data, bit>>
          sendAckQueue = <<>>,    \* public channel <<bit>>
          mess = Message,    
          sender_bit = 0,
          read_state = TRUE;  \* ghost variable
fair+  process sendData = 11 
begin  
  sSd:
  while Len(mess) > 0  do 
   sSdata:
   await (Len(sendDataQueue) = 0) ;
   if (Len(mess) > 0) then
     sendDataQueue := <<[data|->Head(mess), bit|->sender_bit]>>;     
   end if;
  end while;   
end process;

fair+  process sendAck = 12 
begin  
  sSa:
  while Len(mess) > 0  do 
   sSack:
   await ( Len(sendAckQueue) >0) ;
   if (Len(mess) > 0) then
     if ( sender_bit = Head(sendAckQueue)) then
         mess := Tail(mess);   
         read_state := TRUE;         
         if (sender_bit = 0) then sender_bit := 1;
         else sender_bit := 0;   
         end if ;
     end if;         
     sendAckQueue := Tail(sendAckQueue);  
   end if;        
  end while;   
end process;



end algorithm; *)


\* BEGIN TRANSLATION
VARIABLES sendDataQueue, sendAckQueue, mess, sender_bit, read_state, pc

vars == << sendDataQueue, sendAckQueue, mess, sender_bit, read_state, pc >>

ProcSet == {11} \cup {12}

Init == (* Global variables *)
        /\ sendDataQueue = <<>>
        /\ sendAckQueue = <<>>
        /\ mess = Message
        /\ sender_bit = 0
        /\ read_state = TRUE
        /\ pc = [self \in ProcSet |-> CASE self = 11 -> "sSd"
                                        [] self = 12 -> "sSa"]

sSd == /\ pc[11] = "sSd"
       /\ IF Len(mess) > 0
             THEN /\ pc' = [pc EXCEPT ![11] = "sSdata"]
             ELSE /\ pc' = [pc EXCEPT ![11] = "Done"]
       /\ UNCHANGED << sendDataQueue, sendAckQueue, mess, sender_bit, 
                       read_state >>

sSdata == /\ pc[11] = "sSdata"
          /\ (Len(sendDataQueue) = 0)
          /\ IF (Len(mess) > 0)
                THEN /\ sendDataQueue' = <<[data|->Head(mess), bit|->sender_bit]>>
                ELSE /\ TRUE
                     /\ UNCHANGED sendDataQueue
          /\ pc' = [pc EXCEPT ![11] = "sSd"]
          /\ UNCHANGED << sendAckQueue, mess, sender_bit, read_state >>

sendData == sSd \/ sSdata

sSa == /\ pc[12] = "sSa"
       /\ IF Len(mess) > 0
             THEN /\ pc' = [pc EXCEPT ![12] = "sSack"]
             ELSE /\ pc' = [pc EXCEPT ![12] = "Done"]
       /\ UNCHANGED << sendDataQueue, sendAckQueue, mess, sender_bit, 
                       read_state >>

sSack == /\ pc[12] = "sSack"
         /\ ( Len(sendAckQueue) >0)
         /\ IF (Len(mess) > 0)
               THEN /\ IF ( sender_bit = Head(sendAckQueue))
                          THEN /\ mess' = Tail(mess)
                               /\ read_state' = TRUE
                               /\ IF (sender_bit = 0)
                                     THEN /\ sender_bit' = 1
                                     ELSE /\ sender_bit' = 0
                          ELSE /\ TRUE
                               /\ UNCHANGED << mess, sender_bit, read_state >>
                    /\ sendAckQueue' = Tail(sendAckQueue)
               ELSE /\ TRUE
                    /\ UNCHANGED << sendAckQueue, mess, sender_bit, read_state >>
         /\ pc' = [pc EXCEPT ![12] = "sSa"]
         /\ UNCHANGED sendDataQueue

sendAck == sSa \/ sSack

Next == sendData \/ sendAck
           \/ (* Disjunct to prevent deadlock on termination *)
              ((\A self \in ProcSet: pc[self] = "Done") /\ UNCHANGED vars)

Spec == /\ Init /\ [][Next]_vars
        /\ WF_vars(Next)
        /\ SF_vars(sendData)
        /\ SF_vars(sendAck)

Termination == <>(\A self \in ProcSet: pc[self] = "Done")

\* END TRANSLATION


=============================================================================
\* Modification History
\* Last modified Sun Feb 17 10:01:25 NZDT 2019 by dstr
\* Created Thu Feb 14 15:38:49 NZDT 2019 by dstr
